//
//  Filename:	QFoldout.cpp
//	Created:	17:3:2008   19:36

/*
=========================================================================================

			HEADERS & DEFS

=========================================================================================
*/

#include	<algorithm>
#include	"QFoldout.h"

/*
=========================================================================================

			CODE

=========================================================================================
*/

// ** QFoldout::QFoldout
QFoldout::QFoldout( QWidget *parent ) : QWidget( parent )
{
	if( parent ) {
		resize( QSize( parent->width() - 20, parent->height() ) );
	}

	buttonMap.clear();
	categoryButtons.clear();
	categoryGroups.clear();

	scroll = new QScrollArea( parent );
	scroll->setWidget( this );
	scroll->setMaximumSize( maximumSize() );
	scroll->setMinimumSize( minimumSize() );
	scroll->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
	scroll->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOn );

	if( parent ) {
		scroll->resize( parent->size() );
	}
}

// ** QFoldout::AddPage
void QFoldout::AddPage( const std::string& name, const std::string& caption, QGroupBox *group )
{
	QPushButton *button = new QPushButton( caption.c_str(), this );

	connect( button, SIGNAL( clicked(bool) ), this, SLOT( OnCategoryButtonClicked(bool) ) );

	button->setCheckable( true );
	button->setChecked( true );
	button->setVisible( true );
	group->setVisible( true );
	group->setParent( this );

	categoryButtons.push_back( button );
	categoryGroups.push_back( group );
	buttonMap[ name ] = button;
	groupBoxMap[ name ] = group;

	Layout();
}

// ** QFoldout::RemovePage
void QFoldout::RemovePage( const std::string& name )
{
	QPushButton *button = GetPageButton( name );
	QGroupBox	*group	= GetPageGroup( name );

	if( !button || !group ) {
		return;
	}

	group->setVisible( false );

	buttonMap.erase( name );
	groupBoxMap.erase( name );

	categoryButtons.erase( std::find( categoryButtons.begin(), categoryButtons.end(), button ) );
	categoryGroups.erase( std::find( categoryGroups.begin(), categoryGroups.end(), group ) );

	delete button;

	Layout();
}

// ** QFoldout::OnCategoryButtonClicked
void QFoldout::OnCategoryButtonClicked( bool checked )
{
	QPushButton *button = ( QPushButton* )sender();

	uint n = categoryButtons.size();

	while( n-- ) {
		if( categoryButtons[n] == button ) {
			categoryGroups[n]->setVisible( checked );
			break;
		}
	}

	Layout();
}

// ** QFoldout::Layout
void QFoldout::Layout( void )
{
	int w = width();
	int yOffset = 0;

	const int k_ButtonHeight = 22;
	const int k_ButtonOffset = 0;

	uint n = categoryButtons.size();

	for( int i = 0; i < n; i++ ) {
		QPushButton *button = categoryButtons[i];
		QGroupBox	*group	= categoryGroups[i];

		button->setGeometry( k_ButtonOffset, yOffset, w - k_ButtonOffset * 2, k_ButtonHeight );
		group->setGeometry( 0, yOffset + k_ButtonHeight, w, group->height() );

		if( button->isVisible() ) yOffset += k_ButtonHeight;
		if( group->isVisible() ) yOffset += group->height();
	}

	resize( QSize( w, yOffset ) );
}

// ** QFoldout::IsPageVisible
bool QFoldout::IsPageVisible( const std::string& name ) const {
	QPushButton *button = GetPageButton( name );
	return ( button ) ? button->isVisible() : false;
}

// ** QFoldout::SetPageVisible
void QFoldout::SetPageVisible( const std::string& name, bool visible )
{
	QPushButton *button = GetPageButton( name );
	QGroupBox	*group	= GetPageGroup( name );

	if( button && group ) {
		button->setVisible( visible );
		button->setChecked( visible );
		group->setVisible( visible );
	}

	Layout();
}

// ** QFoldout::SetPageOpened
void QFoldout::SetPageOpened( const std::string& name, bool opened )
{
	QPushButton *button = GetPageButton( name );
	QGroupBox	*group	= GetPageGroup( name );

	if( button && group ) {
		button->setChecked( opened );
		group->setVisible( opened );
	}

	Layout();
}

// ** QFoldout::SetPageCaption
void QFoldout::SetPageCaption( const std::string& name, const std::string& caption )
{
	QPushButton *button = GetPageButton( name );

	if( button ) {
		button->setText( caption.c_str() );
	}
}

// ** QFoldout::GetPageButton
QPushButton* QFoldout::GetPageButton( const std::string& pageName ) const
{
	tButtonMapIter btn = buttonMap.find( pageName );

	if( btn != buttonMap.end() ) {
		return btn->second;
	}

	return NULL;
}

// ** QFoldout::GetPageGroup
QGroupBox* QFoldout::GetPageGroup( const std::string& pageName ) const
{
	tGroupBoxMapIter grp = groupBoxMap.find( pageName );

	if( grp != groupBoxMap.end() ) {
		return grp->second;
	}

	return NULL;
}